#include <stdint.h>
#include <stddef.h>

typedef struct vmu_pkg {
    char           desc_short[20];     /**< \brief Short file description */
    char           desc_long[36];      /**< \brief Long file description */
    char           app_id[20];         /**< \brief Application ID */
    int32_t        icon_cnt;           /**< \brief Number of icons */
    int32_t        icon_anim_speed;    /**< \brief Icon animation speed */
    int32_t        eyecatch_type;      /**< \brief "Eyecatch" type */
    int32_t        data_len;           /**< \brief Number of data (payload) bytes */
    uint16_t       icon_pal[16];       /**< \brief Icon palette (ARGB4444) */
    const uint8_t  *icon_data;         /**< \brief 512*n bytes of icon data */
    const uint8_t  *eyecatch_data;     /**< \brief Eyecatch data */
    const uint8_t  *data;              /**< \brief Payload data */
} vmu_pkg_t;

typedef struct vmu_hdr {
    char      desc_short[16];     /**< \brief Space-padded short description */
    char      desc_long[32];      /**< \brief Space-padded long description*/
    char      app_id[16];         /**< \brief Null-padded application ID */
    uint16_t  icon_cnt;           /**< \brief Number of icons */
    uint16_t  icon_anim_speed;    /**< \brief Icon animation speed */
    uint16_t  eyecatch_type;      /**< \brief Eyecatch type */
    uint16_t  crc;                /**< \brief CRC of the file */
    uint32_t  data_len;           /**< \brief Payload size */
    uint8_t   reserved[20];       /**< \brief Reserved (all zero) */
    uint16_t  icon_pal[16];       /**< \brief Icon palette (ARGB4444) */
    /* 512*n Icon Bitmaps */
    /* Eyecatch palette + bitmap */
} vmu_hdr_t;

#define VMUPKG_EC_NONE      0   /**< \brief No eyecatch */
#define VMUPKG_EC_16BIT     1   /**< \brief 16-bit ARGB4444 */
#define VMUPKG_EC_256COL    2   /**< \brief 256-color palette */
#define VMUPKG_EC_16COL     3   /**< \brief 16-color palette */

int vmu_pkg_build(vmu_pkg_t *src, uint8_t ** dst, size_t * dst_size);
